from IPython.display import display, HTML
import pandas as pd

from lhcsmapi.reference.Reference import Reference
from lhcsmapi.dbsignal.ExpSignalAnalysis import ExpSignalAnalysis


class EnergyExtractionAnalysis(object):

    @staticmethod
    def get_min_max_t_res_body_max(t_res_body_dfs):
        t_ref_body_max = [t_res_body_df.max().values[0] for t_res_body_df in t_res_body_dfs]

        return min(t_ref_body_max), max(t_ref_body_max)

    @staticmethod
    def rate_t_res_body(t_res_body_dfs):
        t_res_body_max_min, t_res_body_max_max = EnergyExtractionAnalysis.get_min_max_t_res_body_max(t_res_body_dfs)

        return "OK" if (t_res_body_max_max < 200) and (t_res_body_max_min > 100) else "NOT_OK"

    @staticmethod
    def analyse_energy_extraction(i_meas_df, u_dump_res_1_sync_df, u_dump_res_2_sync_df, t_delay_ee, timestamp_fgc,
                                  checks='all'):
        ee_analysis_df = Reference.get_energy_extraction_reference_parameters(timestamp_fgc)

        index_i_meas_u_dump_res_1_max = i_meas_df.index.get_loc(u_dump_res_1_sync_df['U_DUMP_RES'].idxmax(),
                                                                method='nearest')
        i_meas_u_dump_res_1_max = i_meas_df.loc[i_meas_df.index[index_i_meas_u_dump_res_1_max], 'I_MEAS']

        index_i_meas_u_dump_res_2_max = i_meas_df.index.get_loc(u_dump_res_2_sync_df['U_DUMP_RES'].idxmax(),
                                                                method='nearest')
        i_meas_u_dump_res_2_max = i_meas_df.loc[i_meas_df.index[index_i_meas_u_dump_res_2_max], 'I_MEAS']

        tau_u_dump_res_1 = ExpSignalAnalysis.calculate_char_time(u_dump_res_1_sync_df,
                                                                        t_start_decay=1, t_end_decay=120)
        tau_u_dump_res_2 = ExpSignalAnalysis.calculate_char_time(u_dump_res_2_sync_df,
                                                                        t_start_decay=1, t_end_decay=120)
        res_1_max = u_dump_res_1_sync_df['U_DUMP_RES'].max() / i_meas_u_dump_res_1_max
        res_2_max = u_dump_res_2_sync_df['U_DUMP_RES'].max() / i_meas_u_dump_res_2_max
        value = [res_1_max, res_2_max, t_delay_ee[0] * 1e-9, t_delay_ee[1] * 1e-9, tau_u_dump_res_1,
                 tau_u_dump_res_2]
        ee_analysis_df['act'] = value
        ee_analysis_df['result'] = ee_analysis_df.apply(
            lambda row: (row['act'] >= row['min']) and (row['act'] <= row['max']), axis=1)

        if checks is not 'all':
            ee_analysis_df = ee_analysis_df.loc[checks]

        if ee_analysis_df['result'].all():
            return True
        else:
            print('The following parameters did not satisfy the comparison criteria')
            display(HTML(ee_analysis_df[ee_analysis_df['result'] == False].to_html()))
            return False

    @staticmethod
    def analyse_energy_extraction_temperature(t_res_body_dfs):

        temp_ref_ee = Reference.get_energy_extraction_reference_temperature()

        # check if temperature is within the +/-25 range
        t_res_outside_range = []
        for index, t_res_body_df in enumerate(t_res_body_dfs):
            idx = temp_ref_ee.index.union(t_res_body_df.index)
            temp_ref_ee_analysis = temp_ref_ee.reindex(idx).interpolate('index')
            temp_ref_ee_analysis = temp_ref_ee_analysis.join(t_res_body_df).dropna()
            temp_ref_ee_analysis['Temp_min'] = temp_ref_ee_analysis['Temp'] - 25
            temp_ref_ee_analysis['Temp_max'] = temp_ref_ee_analysis['Temp'] + 25
            signal_name = t_res_body_df.columns[0]
            temp_ref_ee_analysis['Comparison'] = temp_ref_ee_analysis.apply(
                lambda row: (row[signal_name] >= row['Temp_min']) and (row[signal_name] <= row['Temp_max']), axis=1)
            if any(temp_ref_ee_analysis['Comparison'] == False):
                t_res_outside_range.append((index, signal_name))

        if len(t_res_outside_range) > 0:
            print('The following temperatures are outside of the reference profile by more than 25 K:')
            print(t_res_outside_range)

        # check if min_max_temperature
        t_ref_body_max = [t_res_body_df.max().values[0] for t_res_body_df in t_res_body_dfs]
        indices = [t_res_body_df.columns[0] for t_res_body_df in t_res_body_dfs]
        t_res_max_analysis_df = pd.DataFrame(index=indices, data=t_ref_body_max, columns=['T_RES_BODY_act'])
        t_res_max_analysis_df['T_RES_BODY_min'] = 100
        t_res_max_analysis_df['T_RES_BODY_max'] = 200

        t_res_max_analysis_df['Comparison'] = t_res_max_analysis_df.apply(lambda row: (
                row['T_RES_BODY_act'] >= row['T_RES_BODY_min'] and (
                row['T_RES_BODY_act'] <= row['T_RES_BODY_max'])), axis=1)

        if any(t_res_max_analysis_df['Comparison'] == False):
            print('The following maximum temperatures did not satisfy the comparison criteria:')
            display(HTML(t_res_max_analysis_df[t_res_max_analysis_df['Comparison'] == False].to_html()))

        return (len(t_res_outside_range) == 0) and all(t_res_max_analysis_df['Comparison'] == True)
