from lhcsmapi.metadata.SignalMetadata import SignalMetadata

from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
from lhcsmapi.dbsignal.Signal import Signal


class EnergyExtractionQuery(object):

    @staticmethod
    def query_u_dump_res(metadata_ee, timestamp_fgc, circuit_name):
        # Find PM events
        event_source_timestamp_ee = PmDbRequest.find_events('*', metadata_ee['system'], metadata_ee['className'],
                                                            t_start=timestamp_fgc, duration=[(10, 's'), (200, 's')])
        # Filter events from the analyzed circuit
        event_source_timestamp_ee = [elem for elem in event_source_timestamp_ee if circuit_name in elem[0]]

        # Sort events according to EE location - an odd one is triggered first, and the even one afterwards
        event_source_timestamp_ee = sorted(event_source_timestamp_ee, key=lambda source_ts: int(source_ts[0][2]))

        if len(event_source_timestamp_ee) != 2:
            raise ValueError("EE PMs were not found!")

        # metadata
        U_DUMP_RES = "U_DUMP_RES"
        # EE1
        source_ee_1 = event_source_timestamp_ee[0][0]
        timestamp_ee_1 = event_source_timestamp_ee[0][1]
        u_dump_res_1_df = Signal().read('pm', system=metadata_ee['system'], className=metadata_ee['className'],
                                        source=source_ee_1, signal=U_DUMP_RES, eventTime=timestamp_ee_1)
        u_dump_res_1_sync_df = u_dump_res_1_df.set_index(((u_dump_res_1_df.index.values) - timestamp_fgc) / 1e+9)

        # subtract offset
        u_dump_res_1_offset = u_dump_res_1_sync_df[u_dump_res_1_sync_df.index < 0].mean().values[0]
        u_dump_res_1_sync_df = u_dump_res_1_sync_df.subtract(u_dump_res_1_offset)

        # EE2
        source_ee_2 = event_source_timestamp_ee[1][0]
        timestamp_ee_2 = event_source_timestamp_ee[1][1]
        u_dump_res_2_df = Signal().read('pm', system=metadata_ee['system'], className=metadata_ee['className'],
                                        source=source_ee_2, signal=U_DUMP_RES, eventTime=timestamp_ee_2)
        u_dump_res_2_sync_df = u_dump_res_2_df.set_index(((u_dump_res_2_df.index.values) - timestamp_fgc) / 1e+9)

        # subtract offset
        u_dump_res_2_offset = u_dump_res_2_sync_df[u_dump_res_2_sync_df.index < 0].mean().values[0]
        u_dump_res_2_sync_df = u_dump_res_2_sync_df.subtract(u_dump_res_2_offset)

        return u_dump_res_1_sync_df, u_dump_res_2_sync_df, [timestamp_ee_1, timestamp_ee_2]

    @staticmethod
    def query_t_res_body_pm(metadata_ee, T_RES_BODY, timestamp_fgc):
        event_source_timestamp_ee = PmDbRequest.find_events('*', metadata_ee['system'], metadata_ee['className'],
                                                            t_start=timestamp_fgc, duration=[(10, 's'), (500, 's')])
        t_res_body_dfs = []
        for source_ee, timestamp_ee in event_source_timestamp_ee:
            for T_RES_BODY_el in T_RES_BODY:
                t_res_body_df = Signal().read('pm', append=True, system=metadata_ee['system'],
                                              className=metadata_ee['className'], source=source_ee,
                                              signal=T_RES_BODY_el.replace('%EE%', source_ee),
                                              eventTime=timestamp_ee)

                t_res_body_sync_df = t_res_body_df.set_index(((t_res_body_df.index.values) - timestamp_fgc) / 1e+9)
                t_res_body_sync_df = t_res_body_sync_df.drop(t_res_body_sync_df.index[0:200])
                t_res_body_dfs.append(t_res_body_sync_df)
        return t_res_body_dfs

    @staticmethod
    def query_cals(circuit_type, circuit_name, timestamp_fgc, variable_name, ldb):
        system_type = 'EE'
        database_name = 'CALS'

        var_names = SignalMetadata.get_signal_name(circuit_type, circuit_name, system_type, database_name,
                                                   variable_name)

        signal_dfs = []
        for var_name in var_names:
            # Query after
            signal_df = Signal().read('cals', signal=var_name, t_start=(timestamp_fgc, 'ns'),
                                          duration=[(3 * 3600, 's')], ldb=ldb)
            signal_df = signal_df.set_index((signal_df.index.values - timestamp_fgc * 1e-9))
            signal_dfs.append(signal_df)

        return signal_dfs