import matplotlib.pyplot as plt
import pandas as pd
from lhcsmapi.Time import Time


class CurrentLeadsPlot(object):

    @staticmethod
    def plot_leads_voltage(u_leads_df, timestamp_leads, circuit_name, leads_type):
        title = '{} {}: U_HTS and U_RES for {}'.format(Time.to_string_short(timestamp_leads), circuit_name,
                                                       leads_type)

        fig, ax = plt.subplots(1,2)
        fig.suptitle(title, fontsize=20)
        u_leads_df.filter(regex='U_RES').plot(ax=ax[0], figsize=(30, 10), color=['C0', 'C1'], grid=True)

        ax[0].title.set_size(20)
        ax[0].set_xlabel("time, [s]", fontsize=15)
        ax[0].legend(loc='lower left')
        ax[0].set_ylabel("U_RES, [V]", fontsize=15)
        ax[0].tick_params(labelsize=15)
        ax[0].axhspan(ymin=-0.1, ymax=+0.1, facecolor='xkcd:yellowgreen', alpha=0.5)

        u_leads_df.filter(regex='U_HTS').plot(ax=ax[1], color=['C2', 'C3'], marker='o', grid=True)
        ax[1].axhspan(ymin=-0.003, ymax=+0.003, facecolor='xkcd:yellowgreen', alpha=0.5)
        ax[1].legend(loc='lower left')
        ax[1].set_xlabel("time, [s]", fontsize=15)
        ax[1].set_ylabel("U_HTS, [V]", fontsize=15)
        ax[1].tick_params(labelsize=15)
        ax[1].set_xlim([-2, 5])
        plt.show()

    @staticmethod
    def create_plot_title(hwc_test, t_start, t_end, signal):
        return 'HWC test {} start time: {}, end time: {}, {}'.format(hwc_test, t_start, t_end, signal)

    @staticmethod
    def plot_current_leads(i_meas_df, current_lead_dfs, title, ylabel, ylim, ythres):
        ax1 = i_meas_df.plot(figsize=(30, 10), grid=True)
        ax1.set_title(title)
        ax1.title.set_size(20)
        ax1.set_xlabel("time, [s]", fontsize=15)
        ax1.legend(loc='lower left')
        ax1.set_ylabel("I_MEAS, [A]", fontsize=15)
        ax1.tick_params(labelsize=15)

        ax2 = ax1.twinx()

        for current_lead_df in current_lead_dfs:
            current_lead_df.plot(ax=ax2)

        ax2.legend(loc='upper right')
        ax2.set_ylabel(ylabel, fontsize=15)
        ax2.tick_params(labelsize=15)

        if isinstance(ythres, pd.DataFrame):
            ax2.fill_between(ythres.index, ythres['envelope_pos'], ythres['envelope_neg'], color='xkcd:yellowgreen',
                             alpha=0.5)
        else:
            ax2.axhspan(ymin=ythres[0], ymax=ythres[1], facecolor='xkcd:yellowgreen', alpha=0.5)
        ax2.set_ylim(ylim)
        plt.show()