from copy import deepcopy
import scipy as sp
import numpy as np
from IPython.display import display, HTML

from lhcsmapi.analysis.pc.PowerConverterQuery import PowerConverterQuery
from lhcsmapi.dbsignal.ExpSignalAnalysis import ExpSignalAnalysis
from lhcsmapi.reference.Reference import Reference
from lhcsmapi.dbsignal.SignalProcessing import SignalProcessing
from lhcsmapi.Time import Time


class PowerConverterAnalysis(object):

    @staticmethod
    def calculate_fgc_characteristic_time(i_meas_df):
        # Calculate derivative of the actual current
        SignalProcessing.calculate_time_derivative_of_signals(i_meas_df, i_meas_df.index, ['I_MEAS'], ['dI_MEAS'])
        # Calculate time constant as a ratio of a signal by its derivative
        i_meas_df["I_MEAS/dI_MEAS"] = -i_meas_df['I_MEAS'] / sp.signal.medfilt(i_meas_df["dI_MEAS"], 3)
        i_meas_df.loc[i_meas_df.index < 0, ["I_MEAS/dI_MEAS"]] = np.nan
        return i_meas_df

    @staticmethod
    def analyze_fgc_current(metadata_fgc, timestamp_fgc, circuit_type, circuit_name):
        # For the actual fgc timestamp
        # # Query and synchronize
        i_meas_df = PowerConverterQuery.query_preprocess_fgc_current(metadata_fgc, 'I_MEAS', timestamp_fgc)
        # # Calculate characteristic discharge time as I_MEAS/dI_MEAS
        i_meas_tau_df = PowerConverterAnalysis.calculate_fgc_characteristic_time(i_meas_df)

        # For the reference fgc timestamp
        # # Query and synchronize
        rb_pno_b2 = Reference.get_power_converter_reference_fpa(circuit_type)
        timestamp_fgc_ref = Time.to_unix_timestamp(rb_pno_b2[rb_pno_b2['circuit'] == circuit_name]['fgcPm'].values[0])
        i_meas_ref_df = PowerConverterQuery.query_preprocess_fgc_current(metadata_fgc, 'I_MEAS', timestamp_fgc_ref)

        # # Calculate characteristic discharge time as I_MEAS/dI_MEAS
        i_meas_ref_tau_df = PowerConverterAnalysis.calculate_fgc_characteristic_time(i_meas_ref_df)

        return i_meas_tau_df, i_meas_ref_tau_df, timestamp_fgc_ref

    @staticmethod
    def analyze_i_meas_characteristic_time(i_meas_df):
        pc_analysis_df = Reference.gety_power_converter_parameters()

        tau_i_meas = ExpSignalAnalysis.calculate_char_time(i_meas_df, t_start_decay=1, t_end_decay=120)

        pc_analysis_df['act'] = tau_i_meas
        pc_analysis_df['result'] = pc_analysis_df.apply(
            lambda row: (row['act'] >= row['min']) and (row['act'] <= row['max']), axis=1)

        if pc_analysis_df['result'].all():
            return True
        else:
            print('The following parameters did not satisfy the comparison criteria')
            display(HTML(pc_analysis_df[pc_analysis_df['result'] == False].to_html()))
            return False

    @staticmethod
    def is_earth_current_within_ref_envelope(i_earth_pcnt_df, i_earth_pcnt_ref_df, tolerance=3):
        i_earth_pcnt_analysis_df = deepcopy(i_earth_pcnt_df)
        # filter out the spikes
        i_earth_pcnt_analysis_df['I_EARTH_PCNT'] = i_earth_pcnt_analysis_df['I_EARTH_PCNT'].rolling(5).median().values
        i_earth_pcnt_analysis_df['I_EARTH_PCNT_REF_max'] = i_earth_pcnt_ref_df['I_EARTH_PCNT'].values + tolerance
        i_earth_pcnt_analysis_df['I_EARTH_PCNT_REF_min'] = i_earth_pcnt_ref_df['I_EARTH_PCNT'].values - tolerance
        i_earth_pcnt_analysis_df['I_EARTH_PCNT_REF_within_range'] = i_earth_pcnt_analysis_df.apply(lambda row: (
                    row['I_EARTH_PCNT'] >= row['I_EARTH_PCNT_REF_min'] and (
                        row['I_EARTH_PCNT'] <= row['I_EARTH_PCNT_REF_max'])), axis=1)

        return all(
            i_earth_pcnt_analysis_df.loc[i_earth_pcnt_analysis_df.index >= 5, 'I_EARTH_PCNT_REF_within_range'] == True)