import matplotlib.pyplot as plt

from lhcsmapi.Time import Time
from lhcsmapi.analysis.qds.iqds.IqdsPlot import IqdsPlot
from lhcsmapi.analysis.qds.nqds.NqdsPlot import NqdsPlot
from lhcsmapi.metadata.MappingMetadata import MappingMetadata

class QdsPlot(object):

    @staticmethod
    def plot_iqds_nqds(source_qds, timestamp_qds, u_qds_df, timestamp_pic, timestamp_nqps, t_start_quench, t_st_nqd0,
                       t_st_magnet_ok, u_nqps_df, u_nqps_shift_diff_df, i_meas_shift_df, circuit_name, i_quench):
        # # iQPS
        nqps_crate = MappingMetadata.get_crate_name_from_magnet_name(circuit_type='RB', magnet=source_qds)
        title = "Magnet: {}, iQPS Time Stamp: {}, nQPS Crate: {}, Current = {} A".\
            format(source_qds,Time.to_string_short(timestamp_qds), nqps_crate, i_quench)
        fig, ax = plt.subplots(2, 2, figsize=(30, 15))
        fig.suptitle(title, fontsize=20)

        IqdsPlot.plot_u_qds(u_qds_df, timestamp_qds, timestamp_pic, title=title, xmin=t_start_quench, xmax=t_st_nqd0,
                            xlim=(t_start_quench - 0.025, t_st_magnet_ok + 0.075), ax=ax[0, 0])
        IqdsPlot.plot_st_qds(u_qds_df, xmin=t_start_quench, xmax=t_st_nqd0, title=title,
                             xlim=(t_start_quench - 0.025, t_st_magnet_ok + 0.075), ax=ax[1, 0])

        # # nQPS
        if abs(timestamp_nqps - timestamp_qds) < 2e9 and not u_nqps_df.empty:
            NqdsPlot.plot_u_diode_pm_nqds(u_nqps_df, timestamp_nqps, circuit_name, ax=ax[0, 1])

            u_nqps_shift_diff_filt_df = u_nqps_shift_diff_df.filter(regex=source_qds)
            if not u_nqps_shift_diff_filt_df.empty:
                NqdsPlot.plot_u_diode_diff_pm(u_nqps_shift_diff_filt_df, timestamp_nqps, circuit_name,
                                              t_st_magnet_ok, ax=ax[1,1])
            else:
                ax[1, 1].axis('off')
        else:
            NqdsPlot.plot_u_diode_cals(u_nqps_df, timestamp_nqps, circuit_name, i_meas_df=i_meas_shift_df, ax=ax[0, 1])
            NqdsPlot.plot_u_diode_cals(u_nqps_df, timestamp_nqps, circuit_name, i_meas_df=None, ax=ax[1, 1], xlim=[-2, 2])

        ax[0,1].xaxis.set_tick_params(which='both', labelbottom=True)
        plt.show()
