import math
import pandas as pd
import requests
import warnings
from tqdm.notebook import tqdm

import lhcsmapi
from lhcsmapi.metadata.MappingMetadata import MappingMetadata
from lhcsmapi.Time import Time


class IqdsAnalysis(object):

    @staticmethod
    def analyze_qds_trigger(u_qds_df):

        st_qds_sub_df = u_qds_df[(u_qds_df.index > u_qds_df.index[10]) &
                                 (u_qds_df['ST_MAGNET_OK'] == False)]

        if len(st_qds_sub_df) > 0:
            t_st_magnet_ok = st_qds_sub_df.index[0]
            t_st_nqd0 = u_qds_df[(u_qds_df.index > u_qds_df.index[10]) &
                                 (u_qds_df.index < t_st_magnet_ok) &
                                 (u_qds_df['ST_NQD0'] == True)].index[-1]

            t_st_nqd0 = u_qds_df.index[u_qds_df.index.get_loc(t_st_nqd0) + 1]
            u_st_nqd0 = u_qds_df[u_qds_df.index == t_st_nqd0]['U_QS0'].values[0]

            if min(abs(u_qds_df['U_QS0'])) <= 0.01:
                u_qds_before_quench_df = u_qds_df[u_qds_df.index < t_st_nqd0]
                t_start_quench = u_qds_before_quench_df[
                    abs(u_qds_before_quench_df['U_QS0'] - u_qds_before_quench_df['U_QS0'].values[0]) <= 0.01].index[-1]
                t_start_quench = u_qds_df.index[u_qds_df.index.get_loc(t_start_quench) - 1]
            else:
                t_start_quench = 0

            u_start_quench = u_qds_df.loc[
                u_qds_df.index[u_qds_df.index.get_loc(t_start_quench, method='nearest')], 'U_QS0']

            du_dt = (u_st_nqd0 - u_start_quench) / (t_st_nqd0 - t_start_quench)

            t_delay_qh_trigger = -t_st_magnet_ok
            analysis_comment = ''
        else:
            t_st_magnet_ok = u_qds_df.index[-1] - 0.075
            t_st_nqd0 = u_qds_df.index[-1] - 0.075
            t_start_quench = u_qds_df.index[0] + 0.025
            u_start_quench = math.nan
            du_dt = 0
            t_delay_qh_trigger = math.nan
            analysis_comment = 'Seems that the heaters were triggered by nQPS'

        COLUMN_NAMES = ["t_st_magnet_ok [s]", "t_st_nqd0 [s]", "u_st_nqd0 [V]", "t_start_quench [s]", "du_dt [V/s]",
                        "t_delay_qh_trigger [s]", "analysis_comment"]
        value = [t_st_magnet_ok, t_st_nqd0, u_start_quench, t_start_quench, du_dt, t_delay_qh_trigger, analysis_comment]
        return pd.DataFrame(columns=COLUMN_NAMES, data=[value])

    @staticmethod
    def create_report_analysis_template(circuit_type, circuit_name, metadata_qds, event_source_timestamp_qds, i_meas_df,
                                        timestamp_fgc):
        # ToDo: From a csv file
        COLUMN_NAMES = ["Circuit", "magnet", "magnetID", "timeStampPM", "dateTimePM", "qCurrent", "A/B", "nQPS",
                        "iQPS/nQPS", "quenchType", "qPolarity", "dU_QS0/dt", "Rmax [uOhm]", "I_MEAS", "IEARTH", "EE",
                        "EE_TEMP", "qHeater", "vFeelers", 'LEADS_ODD', 'LEADS_EVEN', "comments", "doneBy", "version"]
        report_qds = pd.DataFrame(columns=COLUMN_NAMES)

        for index, elem in enumerate(tqdm(event_source_timestamp_qds, desc='Querying PM')):
            source_qds, timestamp_qds = elem
            t_quench = (timestamp_qds - timestamp_fgc) * 1e-9
            if t_quench > i_meas_df.index[-1]:
                i_quench = i_meas_df['I_MEAS'].values[-1]
                warnings.warn('Quench time beyond PM buffer. Need to query CALS in order to get the quench current!')
            else:
                i_quench = int(i_meas_df.reindex([t_quench], method='nearest', tolerance=.1)['I_MEAS'])

            #ToDo: It would be nice to add this query to the library
            pm_request = "http://pm-api-pro/v2/pmdata?system=QPS&className=" + metadata_qds[
                'className'] + "&source=" + source_qds + "&TimestampInNanos=" + str(
                timestamp_qds) + "&pageSize=1&pageIndex=0"
            resp = requests.get(pm_request)
            board_type = resp.json()["content"][0]["namesAndValues"][1]["value"]
            n_qps = MappingMetadata.get_crate_name_from_magnet_name(circuit_type, source_qds)
            magnet_id = MappingMetadata.get_magnet_id_from_magnet_name(circuit_type, 'MB.{}'.format(source_qds))
            values = [circuit_name, source_qds, magnet_id, timestamp_qds, Time.to_string_short(timestamp_qds), i_quench,
                      board_type, n_qps, '?', '?', '?', 0, 0, '', '', '', '', '', '', '', '', '', '', lhcsmapi.__version__]
            report_qds.loc[len(report_qds.index) + 1] = values

        return report_qds.drop_duplicates('magnet')