from copy import deepcopy
import pandas as pd
import math


class NqdsAnalysis(object):

    @staticmethod
    def synchronize_nqps_to_iqps(u_nqps_df, timestamp_nqps, timestamp_qds):
        u_nqps_shift_df = deepcopy(u_nqps_df)
        if not math.isnan(timestamp_nqps):
            u_nqps_shift_df.index = u_nqps_df.index + (timestamp_nqps - timestamp_qds) * 1e-9
        else:
            u_nqps_shift_df.index = u_nqps_df.index + 10

        return u_nqps_shift_df

    @staticmethod
    def calculate_col_diff(df):
        diff_df = pd.DataFrame()
        for index, col_first in enumerate(df.columns):
            magnet_first = NqdsAnalysis._get_magnet_name(col_first)
            for col_second in df.columns[index + 1:]:
                magnet_second = NqdsAnalysis._get_magnet_name(col_second)
                diff_name = '{}-{}'.format(magnet_first, magnet_second)
                diff_df[diff_name] = abs(df[col_first] - df[col_second])

        return diff_df

    @staticmethod
    def _get_magnet_name(signal_name):
        magnet_name = signal_name.split('.')[1].split(':')[0]
        if 'U_REF' in signal_name:
            return magnet_name + ':U_REF'
        else:
            return magnet_name

    @staticmethod
    def get_max_diff_val_and_idx(u_nqps_shift_diff_df, t_st_magnet_ok, source_qds, quenched_magnets):
        # take only certain period of time
        # # a fix in case t_
        u_nqps_diff_slice_df = u_nqps_shift_diff_df[
            (u_nqps_shift_diff_df.index > t_st_magnet_ok-0.2) & (u_nqps_shift_diff_df.index < t_st_magnet_ok)]
        # take only differences with a quenched magnet
        u_nqps_diff_slice_df = u_nqps_diff_slice_df.filter(regex=source_qds)
        if u_nqps_diff_slice_df.empty:
            return -math.inf, 0

        # take only differences for magnets that have not yet quenched
        u_nqps_diff_no_quench_df = pd.DataFrame()
        for col in u_nqps_diff_slice_df.columns:
            if 'U_REF' in col or not all([quenched_magnet in col for quenched_magnet in quenched_magnets]):
                u_nqps_diff_no_quench_df[col] = u_nqps_diff_slice_df[col]
        # take the one above 1V threshold
        max_idx = u_nqps_diff_no_quench_df.max(axis=1).idxmax()
        max_val = u_nqps_diff_no_quench_df.max(axis=0).max()
        return max_idx, max_val
