import matplotlib.pyplot as plt

from lhcsmapi.Time import Time


class NqdsPlot(object):

    @staticmethod
    def plot_u_diode_pm_nqds(u_nqps_df, timestamp_nqps, circuit_name, ax=None):

        if ax is None:
            title = '{} {} U_DIODE_RB(t) and ST_MAGNET_OK_SQRB(t) - PM'.format(
                Time.to_string_short(timestamp_nqps), circuit_name)
            ax1 = u_nqps_df.plot(figsize=(15, 7), title=title, grid=True)
        else:
            ax1 = u_nqps_df.plot(ax=ax, title='nQPS', grid=True)

        ax1.title.set_size(20)
        ax1.legend(loc='lower left')
        ax1.set_xlabel("time, [s]", fontsize=15)
        ax1.set_ylabel("U_DIODE_RB, [V]", fontsize=15)
        ax1.tick_params(labelsize=15)

        if ax is None:
            plt.show()

    @staticmethod
    def plot_u_diode_diff_pm(u_nqps_diff_df, timestamp_nqps, circuit_name, t_st_magnet_ok, ax=None):
        if ax is None:
            title = '{} {} U_DIODE_RB(t) - CALS'.format(Time.to_string_short(timestamp_nqps), circuit_name)
            ax1 = u_nqps_diff_df.plot(figsize=(15, 7), title=title)
        else:
            ax1 = u_nqps_diff_df.plot(ax=ax, title='nQPS', grid=True)

        ax1.axvspan(xmin=-0.2, xmax=t_st_magnet_ok, ymin=0, ymax=0.5, facecolor='xkcd:yellowgreen')
        ax1.set_ylabel("U (Calculated diode differences), [V]", fontsize=15)
        ax1.legend(loc='lower left')
        ax1.set_xlabel("time, [s]", fontsize=15)
        ax1.set_xlim(t_st_magnet_ok-0.2, t_st_magnet_ok+0.1)
        ax1.set_ylim(0, 2)
        ax1.tick_params(labelsize=15)

    @staticmethod
    def plot_u_diode_cals(u_nqps_df, timestamp_nqps, circuit_name, i_meas_df=None, ax=None, xlim=None):

        if ax is None:
            title = '{} {} U_DIODE_RB(t) - CALS'.format(Time.to_string_short(timestamp_nqps), circuit_name)
            ax1 = u_nqps_df.plot(figsize=(15, 7), title=title)
        else:
            ax1 = u_nqps_df.plot(ax=ax, title='nQPS', grid=True)

        ax1.title.set_size(20)
        ax1.legend(loc='lower left')
        ax1.set_xlabel("time, [s]", fontsize=15)
        ax1.set_ylabel("U_DIODE_RB, [V]", fontsize=15)
        ax1.tick_params(labelsize=15)

        if xlim is not None:
            ax1.set_xlim(xlim)

        if i_meas_df is not None:
            ax2 = ax1.twinx()
            i_meas_df['I_MEAS'].plot(ax=ax2, color='C4')
            ax2.legend(loc='upper right')
            ax2.tick_params(labelsize=15)
            ax2.set_ylabel("I_MEAS, [A]", color='C4', fontsize=15)

        ax1.grid(True)

        if ax is None:
            plt.show()