from lhcsmapi.metadata.MappingMetadata import MappingMetadata
from lhcsmapi.metadata.SignalMetadata import SignalMetadata
from lhcsmapi.dbsignal.SignalUtilities import SignalUtilities
from lhcsmapi.dbsignal.Signal import Signal


class NqdsQuery(object):

    @staticmethod
    def query_u_diode_st_magnet_ok_pm_nqds(circuit_type, circuit_name, source_nqps, timestamp_nqps):
        U_DIODE_RB = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'DIODE_{}'.format(circuit_type), 'PM',
                                                    'U_DIODE_{}'.format(circuit_type))
        U_REF_N1 = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'DIODE_{}'.format(circuit_type), 'PM',
                                                  'U_REF_N1')
        metadata_diode = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_name,
                                                                           'DIODE_{}'.format(circuit_type), 'PM')

        magnet_names = MappingMetadata.get_magnet_names_from_crate_name(circuit_type, source_nqps)

        # U_DIODE
        u_nqps_signal = Signal()
        for magnet_name in magnet_names:
            u_nqps_signal.read('pm', append=True, system=metadata_diode['system'], className=metadata_diode['className'],
                               source=source_nqps,
                               signal=U_DIODE_RB.replace('%MAGNET%', magnet_name), eventTime=timestamp_nqps)
        try:
            u_nqps_signal.read('pm', append=True, system=metadata_diode['system'], className=metadata_diode['className'],
                               source=source_nqps,
                               signal=U_REF_N1.replace('%QPS_CRATE%', source_nqps).replace('%CIRCUIT%', circuit_name),
                               eventTime=timestamp_nqps)
        except:
            print('U_REF_N1 signal not present for crate {}'.format(source_nqps))

        u_nqps_df = u_nqps_signal.get_synchronized_time_df(timestamp_nqps)
        u_nqps_df = SignalUtilities.convert_indices_to_sec(u_nqps_df)

        return u_nqps_df

    @staticmethod
    def query_u_diode_cals_nqds(circuit_type, circuit_name, source_nqps, timestamp_nqps, ldb):
        U_DIODE_RB = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'DIODE_{}'.format(circuit_type), 'CALS',
                                                    'U_DIODE_{}'.format(circuit_type))
        U_REF_N1 = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'DIODE_{}'.format(circuit_type), 'CALS',
                                                  'U_REF_N1')

        magnet_names = MappingMetadata.get_magnet_names_from_crate_name(circuit_type, source_nqps)

        # U_DIODE
        u_nqps_signal = Signal()
        for magnet_name in magnet_names:
            u_nqps_signal.read('cals', append=True, signal=U_DIODE_RB.replace('%MAGNET%', magnet_name),
                               t_start=(timestamp_nqps, 'ns'), duration=[(10, 's'), (350, 's')], ldb=ldb)
        try:
            u_nqps_signal.read('cals', append=True,
                               signal=U_REF_N1.replace('%QPS_CRATE%', source_nqps).replace('%CIRCUIT%', circuit_name),
                               t_start=(timestamp_nqps, 'ns'), duration=[(10, 's'), (350, 's')], ldb=ldb)
        except:
            print('U_REF_N1 signal not present for crate {}'.format(source_nqps))

        return u_nqps_signal.get_synchronized_time_df(timestamp_nqps * 1e-9)

    @staticmethod
    def query_nqps_pm_cals(circuit_type, circuit_name, source_nqps, timestamp_nqps, timestamp_qds, ldb):
        if abs(timestamp_nqps - timestamp_qds) < 2e9:
            try:
                return NqdsQuery.query_u_diode_st_magnet_ok_pm_nqds(circuit_type, circuit_name,
                                                                    source_nqps, timestamp_nqps)
            except:
                print(f'Could not query PM for {source_nqps} at {timestamp_nqps}, querying CALS instead')
                return NqdsQuery.query_u_diode_cals_nqds(circuit_type, circuit_name, source_nqps, timestamp_qds,
                                                         ldb=ldb)
        else:
            return NqdsQuery.query_u_diode_cals_nqds(circuit_type, circuit_name, source_nqps, timestamp_qds, ldb=ldb)