import numpy as np

from lhcsmapi.Division import Division


class SignalElectrical:
    """SignalElectrical class contains static methods for calculation of resistance from voltage and current dbsignal
    """
    
    @staticmethod
    def calculate_resistances(df, U_NAMES, I_NAMES, R_NAMES):
        """Method calculates and appends resistance values to passed DataFrame based on the passed uNames and iNames"""
        # Following: https://stackoverflow.com/questions/26886653/pandas-create-new-column-based-on-values-from-other-columns
        for U_NAME, I_NAME, R_NAME in zip(U_NAMES, I_NAMES, R_NAMES):
            col = df.apply(lambda row: Division.divide(row[U_NAME], row[I_NAME]), axis=1)  # get column data with an index
            df = df.assign(**{R_NAME: col.values})  # assign values to column 'c'
        return df

    @staticmethod
    def filter_resistances_for_current_below_threshold(df, I_NAMES, R_NAMES, current_threshold_in_amps=1):
        """Method filters alle resistance values for current values below a certain value
        """
        for I_NAME, R_NAME in zip(I_NAMES, R_NAMES):
            select_indices = list(np.where(abs(df[I_NAME]) <= current_threshold_in_amps)[0])
            df[R_NAME].iloc[select_indices] = 0
    
 