import numpy as np
import pandas as pd
import datetime as dt
#self made functions
import utils as u

class EditDF():
    
    def read_multiple_csv(csv_names):
        """ Function combines different csv and returns DF """
        r = pd.read_csv(csv_names[0] + '.csv', index_col='Unnamed: 0')
        for c in range(1, len(csv_names)):
            new = pd.read_csv(csv_names[c] + '.csv', index_col='Unnamed: 0')
            r = pd.concat([r, new], axis=1, join='inner')
        return r
    
    def drop(resistance, drop_dates):
        """ Function drops date column from the dataframe """
        for d in drop_dates:
            resistance  = resistance.drop(d, axis=1) 
        return resistance
    
    def parse_index(resistance):
        """ Function parses index for sector """
        a = sorted(resistance.index.values, key=lambda x: (x[-9], x[-7], x[-13:-10]))
        resistance = resistance.reindex(index=a)      
        return resistance
    
    def time(big_signals, typ, offset=0):
        """ Function builds chosen type of time vektor """        
        if (typ == 'date') or (typ == 'offset'):
            date = big_signals.columns.values
            date = [pd.Timestamp(date[i]) for i in range(len(date))]    
            if typ == 'offset':
                t = [str(pd.Timestamp(d) + pd.DateOffset(minutes=offset)) for d in date]
                return t
            else:
                return date
        elif typ == 'years': 
            x = big_signals.columns.values
            x = [u.datetime2unixtime_in_second(pd.Timestamp(x[i])) for i in range(len(x))]
            x[:] = [t - x[0] for t in x]
            x[:] = [t/(3600*24*365) for t in x]            
            return x
        
        elif typ == 'days': 
            x = big_signals.columns.values
            x = [u.datetime2unixtime_in_second(pd.Timestamp(x[i])) for i in range(len(x))]
            x[:] = [t - x[0] for t in x]
            x[:] = [t/(3600*24) for t in x]
            return x
        else: 
            print(typ + ' is not a supported type')
            
    def time_from_bmode(b_mode, time): #in order to get other timestamps than 6.0
        """ Function returns important ramp times""" 
        trigger_low = pd.Timestamp(b_mode.iloc[1][time])
        trigger_high = pd.Timestamp(b_mode.iloc[2][time])
        start =  trigger_low + pd.DateOffset(minutes=-30)
        end = trigger_high + pd.DateOffset(minutes=30)
        return (start,trigger_low,trigger_high,end)
    
    def refresh(big_signals, filtered_signal):
        """ Function refreshes Dataframe if format is right""" 
        if str(type(filtered_signal.result)) == "<class 'pandas.core.frame.DataFrame'>":
            print('Signals refreshed')
            return filtered_signal.result
        else:
            print('Signals not refreshed')
            return big_signals
        
    def small_value_dates(snr, margin):
        """ Function returns all dates where the value is smaller than the margin """ 
        cut=[]
        for i in range(len(snr.values)):
            if float(snr.values[i]) <= margin:
                cut.append(snr.index[i])
        return cut

        