import sys
import os
from pathlib import Path

curr_dir = Path(os.path.split(os.getcwd())[0])
utilities_dir = str(curr_dir.parent / 'utilities')

if utilities_dir not in sys.path:
    sys.path.append(utilities_dir)

# Dataframe filtering with widgets
import numpy as np
import pandas as pd
import datetime as dt
# self made functions
import utils as u
import signalCALS as scals
# https://github.com/rdemaria/pytimber
import pytimber

ldb = pytimber.LoggingDB()
import ipywidgets as widgets


class FilterToolbox():

    def import_filter_widgets(resistance):
        """ Function returns preset widgets """
        filt = widgets.SelectMultiple(
            options=['Location: Sector', 'Location: R bigger than'],
            value=['Location: R bigger than'],
            description='F0ilter',
            disabled=False)

        start_time = widgets.Dropdown(
            options=list(resistance.columns.values),
            value=resistance.columns.values[0],
            description='Start Time:',
            disabled=False)

        end_time = widgets.Dropdown(
            options=list(resistance.columns.values),
            value=resistance.columns.values[-1],
            description='End Time:',
            disabled=False)

        sector = widgets.Dropdown(
            options=['all', '12', '23', '34', '45', '56', '67', '78', '81'],
            value='all',
            description='Sector:',
            disabled=False)

        feature = widgets.Dropdown(
            options=[r"R [nOhm]", r"dR/dt [nOhm/year]", r"SNR"],
            value=r"R [nOhm]",
            description='Feature:',
            disabled=False, )

        feature_margin = widgets.FloatText(
            value=11.5,
            description=r"Margin Value:", )
        return filt, start_time, end_time, sector, feature, feature_margin

    def filter_all(resistance, location_parameter, start_time, end_time, sector, feature, feature_margin):
        """ Function filters Dataframe with widgets """
        res = resistance
        # TIME
        res = FilterToolbox.time_range(res, start_time, end_time)
        # LOCATION
        if sector != 'all':
            res = FilterToolbox.sector(res, sector)
        # FEATURE
        if feature == r"R [nOhm]":
            res = FilterToolbox.bigger_than(res, res[res.columns.values[0]],
                                            float(feature_margin) * 1e-9)  # Dataframe, guide Data, margin
        elif feature == r"SNR":
            res = FilterToolbox.bigger_than(res, location_parameter[location_parameter.columns.values[0]],
                                            float(feature_margin))
        elif feature == r"dR/dt [nOhm/year]":
            res = FilterToolbox.bigger_than(res, location_parameter[location_parameter.columns.values[1]],
                                            float(feature_margin) * 1e-9)

        filter_t = FilterToolbox.filter_text(resistance, res, start_time, end_time, sector, feature, feature_margin)

        return res, filter_t

    def filter_text(resistance, res, start_time, end_time, sector, feature, feature_margin):
        filter_t = 'Filter settings:' \
                  + '\n' + "Chosen time period: " \
                  + '\n' + str(start_time) + "  to  " + str(end_time) \
                  + '\n' + "Chosen sector: " + str(sector) \
                  + '\n' + "Chosen feature:" \
                  + '\n' + "All values of " + str(feature) + " bigger than: " + str(feature_margin) \
                  + '\n' + " -> " + str(
            100 * (len(res.index.values) / len(resistance.index.values))) + '% of all Signals have been taken ' + '\n'

        return filter_t

    def time_range(resistance, start_time, end_time):
        trange = resistance.loc[:, start_time:end_time]
        return trange

    def sector(resistance, s):
        """ Function returns chosen sector as DF """
        like_l = s[0] + '.R'
        like_r = s[1] + '.L'
        sector_l = resistance.filter(like=like_l, axis=0)
        sector_r = resistance.filter(like=like_r, axis=0)
        sector = pd.concat([sector_l, sector_r])
        return sector

    def bigger_than(resistance, r_one, margin):
        """ Function returns Signals bigger than margin"""
        r_one_big = r_one[r_one.values > margin]
        index_of_big_signals = r_one_big.index.values
        big_signals = resistance[resistance.index.isin(index_of_big_signals)]
        return big_signals

    def smaller_than(resistance, margin):
        """ Function returns Signals smaller than margin"""
        r_one = resistance[resistance.columns.values[0]]
        r_one_big = r_one[r_one.values < margin]
        index_of_big_signals = r_one_big.index.values
        big_signals = resistance[resistance.index.isin(index_of_big_signals)]
        return big_signals

    def match_current(voltage_name):
        """ Function finds current at certain location """
        current_name = 'RPHE%' + voltage_name[3] + '.A'
        if voltage_name[-7] == 'R':
            current_name = current_name + voltage_name[-9] + '%:I_MEAS'
        elif voltage_name[-7] == 'L':
            if int(voltage_name[-9]) == 8:
                current_name = current_name + str(1) + '%:I_MEAS'
            else:
                current_name = current_name + str(int(voltage_name[-9]) + 1) + '%:I_MEAS'
        current_name = ldb.search(current_name)
        if current_name == []:
            print('could not generate current signal')
        else:
            return current_name[0]
