import sys
import os
from pathlib import Path

curr_dir = Path(os.path.split(os.getcwd())[0])
utilities_dir = str(curr_dir.parent / 'utilities')

if utilities_dir not in sys.path:
    sys.path.append(utilities_dir)

import numpy as np
import pandas as pd
import datetime as dt
# self made functions
import utils as u
import signalCALS as scals
import editDF as edf

# scipy imports
import scipy as sc
from scipy import stats
from scipy.stats import norm


class SignalAnalysis():

    def get_signal_parts(b_mode={}, trigger_low={}, location={}, trigger_high={}):
        """ Function returns the full signaland the signal before and after the ramps """
        if trigger_high == {}:
            (start, trigger_low, trigger_high, end) = edf.EditDF.time_from_bmode(b_mode, trigger_low)
        else:
            start = pd.Timestamp(trigger_low) + pd.DateOffset(minutes=-30)
            end = pd.Timestamp(trigger_high) + pd.DateOffset(minutes=30)
        v_df = scals.Signal(location, start, end).query_cals_signal(location)
        low_voltage = scals.Signal(location, start, trigger_low).query_cals_signal(location)
        high_voltage = scals.Signal(location, trigger_high, end).query_cals_signal(location)
        return (v_df, low_voltage, high_voltage)

    def lin_reg(signal, voltage_name):
        """ Function returns tau_lin_reg function and derivertive of given Dataset"""
        v = signal.loc[voltage_name].values
        v[np.isinf(v)] = 0
        x = edf.EditDF.time(signal, 'years')
        res = stats.mstats.theilslopes(v, x, 0.95)
        x = [res[1] + res[0] * value for value in x]
        return x, res[0]

    def snr(array):
        """ calculates the signal to noise ratio"""
        (mu, sigma) = norm.fit(array)
        snr = abs(mu) / abs(sigma)
        return snr

    def reject_outliers(data, m=4):
        """ rejects data which is outside m *sigma """
        return data[abs(data - np.mean(data)) < m * np.std(data)]

    def percentage(a, b):
        """ returns ratio in percentage """
        return 100 * float(a) / float(b)
