from ipywidgets import HBox, Button, Label, IntProgress, Layout
import functools

from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
from lhcsmapi.Time import Time
from lhcsmapi.gui.pc.FgcBrowserAction import FgcBrowserAction
from lhcsmapi.metadata.SignalMetadata import SignalMetadata
from lhcsmapi.gui.BaseModule import BaseModule


class FgcPmSearchBaseModule(BaseModule):

    def __init__(self):
        self.find_btn = Button(
            description='Find FGC PM entries',
            tooltip='Click to find FGC PM entries between indicated dates.',
            button_style='',  # 'success', 'info', 'warning', 'danger' or ''
        )
        self.find_btn.on_click(functools.partial(FgcPmSearchBaseModule.handle_find_button_event, self=self))

        # # Text field
        self.lbl_search = Label(value="Click button to search for FGC PM entries within the specified period of time")

        # # Progress bar
        self.progress_search = IntProgress(value=0, min=0, step=1, bar_style='success',
                                           orientation='horizontal', layout=Layout(width='20%', visibility='hidden'))

        self.widget = HBox(children=[self.find_btn, self.lbl_search, self.progress_search])

    def handle_find_button_event(b, self):
        self.mediator.notify(self, FgcBrowserAction.FIND_BUTTON_CLICKED)

    def find_fgc_pm_entries(self, start_time, end_time, circuit_type, circuit_name, is_qps):

        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_name, 'PC', 'PM')

        quotient, remainder = Time.modulo_day_in_unix_timestamp(start_time, end_time)
        event_duration_nsec = int(24 * 60 * 60 * 1e9)
        source_timestamp_fgc = []

        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = quotient
        self.progress_search.description = f'Days: 0/{quotient}'

        for day in range(quotient):
            start_time_temp = Time.to_unix_timestamp(start_time) + int(day * event_duration_nsec)

            # Update GUI
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.to_string(start_time_temp))

            source_timestamp_fgc += PmDbRequest.find_events(metadata_fgc['source'], metadata_fgc['system'],
                                                            metadata_fgc['className'], t_start=start_time_temp,
                                                            duration=[(event_duration_nsec, 'ns')])
            self.progress_search.description = f'Days: {day + 1}/{quotient}'
            self.progress_search.value = day + 1

        if remainder > 0:
            start_time_temp = Time.to_unix_timestamp(end_time) - remainder
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.to_string(start_time_temp))
            source_timestamp_fgc += PmDbRequest.find_events(metadata_fgc['source'], metadata_fgc['system'],
                                                            metadata_fgc['className'], t_start=start_time_temp,
                                                            duration=[(remainder, 'ns')])
        # Check if an FGC timestamp has a QPS discharge around
        source_timestamp_fgc_with_qps = []
        metadata_qds = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_name, 'QDS', 'PM')

        for index, elem in enumerate(source_timestamp_fgc):
            # Update GUI
            self.lbl_search.value = 'Checking if there are QPS PM around an FGC timestamp {}'.format(
                Time.to_string(elem[1]))
            self.progress_search.description = f'FGC: {index + 1}/{len(source_timestamp_fgc)}'
            self.progress_search.value = index + 1
            source_timestamp_qds = PmDbRequest.find_events('*', metadata_qds['system'], metadata_qds['className'],
                                                           t_start=elem[1],
                                                           duration=[(60, 's'), (300, 's')])
            source_timestamp_qds_filtered = list(
                filter(lambda x: SignalMetadata.get_sector_names_from_components(circuit_type, 'MAGNET', x[0]) in
                                 circuit_name, source_timestamp_qds))

            if is_qps and source_timestamp_qds_filtered:
                source_timestamp_fgc_with_qps.append(elem)
            elif not (is_qps or source_timestamp_qds_filtered):
                source_timestamp_fgc_with_qps.append(elem)

        self.lbl_search.value = 'Query completed.'

        return source_timestamp_fgc_with_qps

    def get_circuit_name(self):
        return self.select_circuit_name_dropdown.value