import os
import pandas as pd
from dateutil import parser

from lhcsmapi.Time import Time


class Reference(object):

    @staticmethod
    def get_quench_heater_reference_discharges():
        selected_path = os.path.dirname(__file__)
        full_path = os.path.join(selected_path, 'qh/ReferenceDischarges.csv')
        return pd.read_csv(full_path, index_col=0)

    @staticmethod
    def get_energy_extraction_reference_temperature():
        selected_path = os.path.dirname(__file__)
        full_path = os.path.join(selected_path, 'ee/refTemperatureEE.csv')
        return pd.read_csv(full_path, index_col=0)

    @staticmethod
    def get_energy_extraction_reference_parameters(timestamp_fgc):
        selected_path = os.path.dirname(__file__)

        date_change = parser.parse('2018-01-01 00:00:00.000+01:00')
        timestamp_fgc_datetime = Time.to_datetime(timestamp_fgc)

        if timestamp_fgc_datetime > date_change:
            full_path = os.path.join(selected_path, 'ee/scalarParametersEE_after_2018_01_01.csv')
        else:
            full_path = os.path.join(selected_path, 'ee/scalarParametersEE_before_2018_01_01.csv')
        return pd.read_csv(full_path, index_col=0)

    @staticmethod
    def gety_power_converter_parameters():
        selected_path = os.path.dirname(__file__)
        full_path = os.path.join(selected_path, 'pc/scalarParametersPC.csv')
        return pd.read_csv(full_path, index_col=0)

    @staticmethod
    def get_current_leads_parameters():
        selected_path = os.path.dirname(__file__)
        full_path = os.path.join(selected_path, 'leads/scalarParametersLEADS.csv')
        return pd.read_csv(full_path, index_col=0)

    @staticmethod
    def get_power_converter_reference_fpa(circuit_type):
        selected_path = os.path.dirname(__file__)
        if circuit_type == 'RB':
            full_path = os.path.join(selected_path, 'pc/RB_Reference_PNO_b2.csv')
        elif circuit_type == 'RQ':
            full_path = os.path.join(selected_path, 'pc/RQ_Reference_PNO_b3.csv')
        else:
            raise ValueError('Circuit type {} not supported!'.format(circuit_type))
        return pd.read_csv(full_path)

    @staticmethod
    def find_qh_timestamp_for_magnet_name_from_reference(qh_reference_discharges, magnet):
        # Look for a timestamp of the reference discharge curve
        if not qh_reference_discharges[qh_reference_discharges['magnetName'] == magnet].empty:
            return qh_reference_discharges[qh_reference_discharges['magnetName'] == magnet]['timestamp'].values[0]
        else:
            return -1
