import unittest
import pandas as pd

from lhcsmapi.analysis.busbar.BusbarResistanceAnalysis import BusbarResistanceAnalysis


class TestBusbarResistanceAnalysis(unittest.TestCase):

    def test_convert_labelled_voltage_into_mean_vector(self):
        # arrange
        u_res_stat_df = pd.DataFrame({'U_RES_name': {224: 'DCBB.A18L2.R:U_RES', 248: 'DCBB.A18L2.R:U_RES'},
         'U_RES_class': {224: 2, 248: 1},
         'U_RES_mean': {224: 1.4875716959750208e-05, 248: 2.950387599501677e-05},
         'U_RES_std': {224: 2.9723028595248446e-05, 248: 3.166960153942594e-05},
         'U_RES_count': {224: 40, 248: 56}})

        # act
        u_res_stat_df_act = BusbarResistanceAnalysis.convert_labelled_voltage_into_mean_vector(u_res_stat_df, prefix='U_RES')

        # assert
        u_res_stat_df_exp = {'U_RES_name': {0: 'DCBB.A18L2.R:U_RES'},
                 'U_RES_class_1': {0: 1},
                 'U_RES_mean_1': {0: 2.950387599501677e-05},
                 'U_RES_std_1': {0: 3.166960153942594e-05},
                 'U_RES_count_1': {0: 56},
                 'U_RES_class_2': {0: 2},
                 'U_RES_mean_2': {0: 1.4875716959750208e-05},
                 'U_RES_std_2': {0: 2.9723028595248446e-05},
                 'U_RES_count_2': {0: 40},
                 'mean_vector': {0: [2.950387599501677e-05, 1.4875716959750208e-05]}}

        self.assertEqual(u_res_stat_df_exp, u_res_stat_df_act.to_dict())